#include "file.h"
#include "simulateur.h"

float t_traitement_tot=0;
char *algo_name[] = {
  "fifo",
  "pcte",
  "tourniquet",
  "pcter"
};

int comparer_par_a(struct processus * p1,struct processus *p2)
{
  if(p1->t_ajoute <p2->t_ajoute)
    return -1;
  if(p1->t_ajoute == p2->t_ajoute)
    return 0;
  return 1;
}

int comparer_par_te(struct processus * p1,struct processus *p2)
{
  if(p1->t_exec <p2->t_exec )
    return -1;
  if(p1->t_exec == p2->t_exec)
    return 0;
  return 1;
}

void
ajouter_par_t(struct file * dest, struct file *source,float t)
{
  struct processus * p;
  
  while( (p = file_peek_how(source,comparer_par_a) ) && 
	 p->t_ajoute <= t )
    {
      file_pop_how(source,comparer_par_a);
      file_enqueue(dest,p);
    }
 
}


/* Voici les possibles choix differentes */
float
fifo(struct file * file,float t)
{
  struct processus * proc;
  
  proc = file_pop_how(file,comparer_par_a);
  t+=proc->t_exec;
  t_traitement_tot+=(t - proc->t_ajoute);
  
  return t;
}

float
pcte(struct file * file,float t)
{
  struct processus * proc;
  
  proc = file_pop_how(file,comparer_par_te);
  t+=proc->t_exec;
  t_traitement_tot+=(t - proc->t_ajoute);
  
  return t;
}

float
tourniquet(struct file * file,float t,float overhead, float quantum)
{
  struct processus * proc;
  float diff;
  
  proc = file_pop_how(file,comparer_par_a);
  diff = MIN(proc->t_exec,quantum);
  t += diff + overhead;
  t_traitement_tot+=(t - proc->t_ajoute);
  
  proc->t_exec-=diff;
  proc->t_ajoute=t;
  
  if(proc->t_exec > 0 )
    file_enqueue(file,proc);

  return t;
}

float
pcter(struct file * file,float t,float overhead, float quantum)
{
  struct processus * proc;
  float diff;
  
  proc = file_pop_how(file,comparer_par_te);
  diff = MIN(proc->t_exec,quantum);
  t += diff + overhead;
  t_traitement_tot+=(t - proc->t_ajoute);
  
  proc->t_exec-=diff;
  proc->t_ajoute=t;
  
  if(proc->t_exec > 0 )
    file_enqueue(file,proc);

  return t;
}


void 
simulateur(struct file * file_entree,
	   int type_de_choix,float overhead, float quantum)
{

  struct file *file_attente=new_file();
    
  float t=0; /* Le temps ... qui s'en vas */
  
  for(;;)
    {
      ajouter_par_t(file_attente,file_entree,t);
      
      if(file_is_empty(file_attente))
	t+=TEMPS_OUASIF;
      else
	{
	  /* Faire le choix */
	  switch(type_de_choix)
	    {
	    case FIFO:
	      t = fifo(file_attente,t);
	      break;
	    case PCTE:
	      t = pcte(file_attente,t);
	      break;
	    case TOURNIQUET:
	      t = 
		tourniquet(file_attente,t,overhead,quantum);
	      break;
	    case PCTER:
	      t = 
		pcter(file_attente,t,overhead,quantum);
	      break;
	    }
	}
 
      /*
	C'est une simulation ... 
	on peut bien s'arreter
      */
      if(file_is_empty(file_entree) && 
	 file_is_empty(file_attente))
	break;
    }

  free(file_attente);
}

